package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.module.base.pojo.enums.GameCategoryEnum;
import com.zanxiang.game.module.manage.pojo.dto.*;
import com.zanxiang.game.module.manage.pojo.params.GameUserRoleListParam;
import com.zanxiang.game.module.manage.pojo.vo.GameUserRoleListVO;
import com.zanxiang.game.module.manage.pojo.vo.GameUserRoleVO;
import com.zanxiang.game.module.manage.pojo.vo.GameUserVO;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.Agent;
import com.zanxiang.game.module.mybatis.entity.GameUserRole;
import com.zanxiang.game.module.mybatis.mapper.GameUserRoleMapper;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-07-01
 * @description : 玩家角色
 */
@Slf4j
@Service
public class GameUserRoleServiceImpl extends ServiceImpl<GameUserRoleMapper, GameUserRole> implements IGameUserRoleService {

    @Autowired
    private IUserService userService;

    @Autowired
    private IGameService gameService;

    @Autowired
    private IGameUserService gameUserService;

    @Autowired
    private ICpService cpService;

    @Autowired
    private IAgentService agentService;

    @Override
    public GameUserRoleDTO getByRoleId(String roleId) {
        GameUserRole gameUserRole = super.getOne(new LambdaQueryWrapper<GameUserRole>().eq(GameUserRole::getRoleId, roleId));
        return BeanUtil.copy(gameUserRole, GameUserRoleDTO.class);
    }

    @Override
    public List<GameUserRoleVO> roleList(Long userId) {
        List<GameUserRole> gameUserRoleList = super.list(new LambdaQueryWrapper<GameUserRole>().eq(GameUserRole::getUserId, userId));
        if (CollectionUtils.isEmpty(gameUserRoleList)) {
            return Collections.emptyList();
        }
        List<GameUserRoleVO> list = new ArrayList<>();
        gameUserRoleList.forEach(role -> {
            GameUserRoleVO gameUserRoleVO = BeanUtil.copy(role, GameUserRoleVO.class);
            if (gameUserRoleVO == null) {
                return;
            }
            //设置游戏名称
            GameDTO gameDTO = gameService.getById(role.getGameId());
            if (gameDTO != null) {
                gameUserRoleVO.setGameName(gameDTO.getName());
                gameUserRoleVO.setGameCategoryName(GameCategoryEnum.getNameByCategory(gameDTO.getCategory()));
            }
            //玩家游戏信息
            GameUserVO gameUserVO = gameUserService.getById(role.getGameUserId());
            if (gameUserVO != null) {
                Agent agent = agentService.getById(gameUserVO.getAgentId());
                if (agent != null) {
                    gameUserRoleVO.setAgentName(agent.getAgentName());
                }
            }
            list.add(gameUserRoleVO);
        });
        return list;
    }

    @Override
    public IPage<GameUserRoleListVO> list(GameUserRoleListParam param) {
        //游戏条件处理
        Map<Long, GameDTO> gameMap = gameService.gameCondition(param.getCpId(), param.getGameId(), param.getGameCategoryId());
        //根据条件, 匹配不到游戏
        if (gameMap != null && gameMap.isEmpty()) {
            return new Page<>();
        }
        //渠道列表
        List<AgentDTO> agentDTOList = agentService.listUserAgent(null, null, param.getChannelId());
        //根据条件, 匹配渠道
        if (CollectionUtils.isEmpty(agentDTOList) && !SecurityUtil.isManager()) {
            return new Page<>();
        }
        //渠道id
        List<Long> agentIdList = agentDTOList.stream().map(AgentDTO::getId).collect(Collectors.toList());
        //管理员可见自然量数据
        if (SecurityUtil.isManager() && param.getChannelId() == null) {
            agentIdList.add(Agent.DEFAULT_AGENT);
        }
        //玩家条件处理
        Map<Long, UserDTO> userMap = userService.userCondition(param.getUserId(), agentIdList, param.getUserName(),
                param.getNickname(), param.getRegIp());
        //根据条件, 匹配不到玩家
        if (userMap != null && userMap.isEmpty()) {
            return new Page<>();
        }
        //相关用户id
        Set<Long> userIdSet = userMap == null ? null : userMap.keySet();
        Map<Long, CpDTO> cpMap = cpService.cpMap();
        Map<Long, AgentDTO> agentMap = agentDTOList.stream().collect(Collectors.toMap(AgentDTO::getId, Function.identity()));
        //执行查询
        return page(param.toPage(), new QueryWrapper<GameUserRole>().lambda()
                .in(CollectionUtils.isNotEmpty(userIdSet), GameUserRole::getUserId, userIdSet)
                .eq(Strings.isNotBlank(param.getServerName()), GameUserRole::getServerName, param.getServerName())
                .eq(Strings.isNotBlank(param.getRoleName()), GameUserRole::getRoleName, param.getRoleName())
                .in(gameMap != null, GameUserRole::getGameId, gameMap != null ? gameMap.keySet() : null)
                .eq(Strings.isNotBlank(param.getOs()), GameUserRole::getOs, param.getOs())
                .gt(Objects.equals(param.getIsRecharge(), Boolean.TRUE), GameUserRole::getRechargeCount, 0)
                .eq(Objects.equals(param.getIsRecharge(), Boolean.FALSE), GameUserRole::getRechargeCount, 0)
                .eq(param.getVipLevel() != null && param.getVipLevel() <= 10, GameUserRole::getRoleVipLevel, param.getVipLevel())
                .gt(param.getVipLevel() != null && param.getVipLevel() > 10, GameUserRole::getRoleVipLevel, param.getVipLevel())
                .ge(param.getBeginDate() != null, GameUserRole::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, GameUserRole::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .ge(param.getRechargeBeginDate() != null, GameUserRole::getLastRechargeTime, param.getRechargeBeginDate() == null ? null : LocalDateTime.of(param.getRechargeBeginDate(), LocalTime.MIN))
                .le(param.getRechargeEndDate() != null, GameUserRole::getLastRechargeTime, param.getRechargeEndDate() == null ? null : LocalDateTime.of(param.getRechargeEndDate(), LocalTime.MAX))
                .orderByDesc(GameUserRole::getCreateTime)
        ).convert(u -> this.toVo(u, userMap, gameMap, cpMap, agentMap));
    }

    private GameUserRoleListVO toVo(GameUserRole gameUserRole, Map<Long, UserDTO> userMap, Map<Long, GameDTO> gameMap,
                                    Map<Long, CpDTO> cpMap, Map<Long, AgentDTO> agentMap) {
        GameUserRoleListVO gameUserRoleListVO = BeanUtil.copy(gameUserRole, GameUserRoleListVO.class);
        if (Objects.isNull(gameUserRoleListVO)) {
            return null;
        }
        UserDTO userDTO = CollectionUtils.isEmpty(userMap) ? null : userMap.get(gameUserRoleListVO.getUserId());
        if (userDTO == null) {
            userDTO = userService.getById(gameUserRoleListVO.getUserId());
        }
        if (userDTO != null) {
            gameUserRoleListVO.setUsername(userDTO.getUsername());
            gameUserRoleListVO.setNickname(userDTO.getNickname());
            gameUserRoleListVO.setRegIp(userDTO.getIp());
            gameUserRoleListVO.setRegTime(userDTO.getCreateTime());
            gameUserRoleListVO.setAgentId(userDTO.getAgentId());
            AgentDTO agentDTO = agentMap.get(userDTO.getAgentId());
            gameUserRoleListVO.setAgentName(agentDTO == null ? null : agentDTO.getAgentName());
        }
        GameDTO gameDTO = CollectionUtils.isEmpty(gameMap) ? null : gameMap.get(gameUserRoleListVO.getGameId());
        if (gameDTO == null) {
            gameDTO = gameService.getById(gameUserRoleListVO.getGameId());
        }
        if (gameDTO != null) {
            gameUserRoleListVO.setGameName(gameDTO.getName());
            gameUserRoleListVO.setCpId(gameDTO.getCpId());
            gameUserRoleListVO.setCpName(cpMap.get(gameDTO.getCpId()) == null ? null : cpMap.get(gameDTO.getCpId()).getCpName());
            gameUserRoleListVO.setGameCategoryId(gameDTO.getCategory());
            gameUserRoleListVO.setGameCategoryName(GameCategoryEnum.getNameByCategory(gameDTO.getCategory()));
        }
        return gameUserRoleListVO;
    }
}
